'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _dot = require('./dot');

var _dot2 = _interopRequireDefault(_dot);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var propTypes = {
  /**
   * Error message for the object, if there is one.
   */
  errorMessage: _react2.default.PropTypes.string,

  /**
   * Field name of the object in the parent object.
   */
  fieldName: _react2.default.PropTypes.string.isRequired,

  /**
   * The add button label
   */
  addLabel: _react2.default.PropTypes.string,

  /**
   * Show the add button
   */
  showAddButton: _react2.default.PropTypes.bool,

  /**
   * The remove label
   */
  removeLabel: _react2.default.PropTypes.string,

  /**
   * Show the container label
   */
  showLabel: _react2.default.PropTypes.bool,

  /**
   * The field should be read only mode
   */
  disabled: _react2.default.PropTypes.bool,

  /**
   *
   */
  autoAddItem: _react2.default.PropTypes.bool,

  /**
   * The label for the field
   */
  label: _react2.default.PropTypes.string,

  /**
   * Each item component
   */
  children: _react2.default.PropTypes.any,

  /**
   * Pass a function that returns the children components for the current item.
   * The inputs of the function will be value and index.
   * This is useful when you want to change the view of a item in the array depending
   * on the current value.
   */
  renderItem: _react2.default.PropTypes.func
};

var defaultProps = {
  addLabel: 'Add',
  removeLabel: 'Remove',
  showLabel: true,
  errorMessages: {},
  autoAddItem: false,
  showAddButton: true
};

var contextTypes = {
  schema: _react2.default.PropTypes.object,
  doc: _react2.default.PropTypes.object,
  onChange: _react2.default.PropTypes.func.isRequired,
  parentFieldName: _react2.default.PropTypes.string,
  errorMessages: _react2.default.PropTypes.object
};

var childContextTypes = {
  parentFieldName: _react2.default.PropTypes.string
};

var ArrayComponent = function (_React$Component) {
  _inherits(ArrayComponent, _React$Component);

  function ArrayComponent(props) {
    _classCallCheck(this, ArrayComponent);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(ArrayComponent).call(this, props));

    _this.addItem = _this.addItem.bind(_this);
    return _this;
  }

  _createClass(ArrayComponent, [{
    key: 'getSchema',
    value: function getSchema() {
      return this.context.schema;
    }
  }, {
    key: 'getLabel',
    value: function getLabel() {
      if (this.props.showLabel === false) return;
      if (this.props.label) return this.props.label;
      return this.getSchema().label(this.getFieldName());
    }
  }, {
    key: 'getFieldName',
    value: function getFieldName() {
      if (this.context.parentFieldName) {
        return this.context.parentFieldName + '.' + this.props.fieldName;
      } else {
        return this.props.fieldName;
      }
    }
  }, {
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        parentFieldName: this.getFieldName()
      };
    }
  }, {
    key: 'getValue',
    value: function getValue() {
      return this.context.doc ? _dot2.default.pick(this.getFieldName(), this.context.doc) : undefined;
    }
  }, {
    key: 'getErrorMessage',
    value: function getErrorMessage() {
      var errorMessages = this.context.errorMessages || {};
      return this.props.errorMessage || errorMessages[this.getFieldName()];
    }
  }, {
    key: 'onValueChange',
    value: function onValueChange(fieldName, newValue) {
      var withoutSelf = fieldName.replace(this.getFieldName() + '.', '');
      var index = withoutSelf.split('.')[0];
      var plainFieldName = withoutSelf.replace(index + '.', '');
      var value = _underscore2.default.clone(this.getValue());

      if (!value) {
        value = [];
      }

      if (!value[index]) {
        value[index] = {};
      }

      value[index][plainFieldName] = newValue;
      this.context.onChange(this.getFieldName(), value);
    }
  }, {
    key: 'addItem',
    value: function addItem() {
      var itemValue = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var newArray = this.getValue();
      if (_underscore2.default.isArray(newArray)) {
        newArray.push(itemValue);
      } else {
        newArray = [itemValue];
      }

      this.context.onChange(this.getFieldName(), newArray);
    }
  }, {
    key: 'removeItem',
    value: function removeItem(index) {
      var value = this.getValue() || [];
      var newArray = _underscore2.default.without(value, value[index]);
      this.context.onChange(this.getFieldName(), newArray);
    }
  }, {
    key: 'renderChildren',
    value: function renderChildren() {
      var _this2 = this;

      var value = this.getValue() || [];
      if (this.props.autoAddItem && !this.props.disabled && value.length === 0) {
        value.push({});
      }
      return value.map(function (item, index) {
        var children = _this2.props.renderItem ? _this2.props.renderItem(item, index) : _this2.props.children;
        // const component = this.renderChildrenComponent(children, index)
        return _this2.renderChildrenItem({ index: index, children: children });
      });
    }
  }, {
    key: 'renderChildrenItem',
    value: function renderChildrenItem(_ref) {
      var _this3 = this;

      var index = _ref.index;
      var children = _ref.children;

      return _react2.default.createElement(
        'div',
        { style: { marginTop: 20, marginBottom: 20, padding: 20 }, key: this.getFieldName() + '.' + index },
        this.renderChildrenItemWithContext({ index: index, children: children }),
        _react2.default.createElement(
          'div',
          { style: { marginTop: 10, textAlign: 'right' } },
          _react2.default.createElement(
            'button',
            { onClick: function onClick() {
                return _this3.removeItem(index);
              } },
            this.props.removeLabel
          )
        )
      );
    }
  }, {
    key: 'renderChildrenItemWithContext',
    value: function renderChildrenItemWithContext(_ref2) {
      var index = _ref2.index;
      var children = _ref2.children;

      return _react2.default.createElement(
        ArrayContextItem,
        { index: index, fieldName: this.getFieldName() },
        children
      );
    }
  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        'div',
        { style: { marginTop: 20 } },
        _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement(
            'b',
            null,
            this.getLabel()
          )
        ),
        _react2.default.createElement(
          'div',
          { style: { color: 'red' } },
          this.getErrorMessage()
        ),
        this.renderChildren(),
        _react2.default.createElement(
          'div',
          { style: { marginTop: 10 } },
          _react2.default.createElement(
            'button',
            { onClick: this.addItem },
            this.props.addLabel
          )
        )
      );
    }
  }]);

  return ArrayComponent;
}(_react2.default.Component);

exports.default = ArrayComponent;


ArrayComponent.propTypes = propTypes;
ArrayComponent.defaultProps = defaultProps;
ArrayComponent.contextTypes = contextTypes;
ArrayComponent.childContextTypes = childContextTypes;

var arrayContextItemPropTypes = {
  children: _react2.default.PropTypes.any,
  index: _react2.default.PropTypes.number.isRequired,
  fieldName: _react2.default.PropTypes.string.isRequired
};

var arrayContextItemChildContextTypes = {
  parentFieldName: _react2.default.PropTypes.string
};

var ArrayContextItem = function (_React$Component2) {
  _inherits(ArrayContextItem, _React$Component2);

  function ArrayContextItem(props) {
    _classCallCheck(this, ArrayContextItem);

    var _this4 = _possibleConstructorReturn(this, Object.getPrototypeOf(ArrayContextItem).call(this, props));

    _this4.state = {};
    return _this4;
  }

  _createClass(ArrayContextItem, [{
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        parentFieldName: this.props.fieldName + '.' + this.props.index
      };
    }
  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        'div',
        null,
        this.props.children
      );
    }
  }]);

  return ArrayContextItem;
}(_react2.default.Component);

ArrayContextItem.propTypes = arrayContextItemPropTypes;
ArrayContextItem.childContextTypes = arrayContextItemChildContextTypes;